@@ -1,5 +1,5 @@
 /*
- *  $Id: MSR-Athlon.c,v 1.3 2001/12/10 20:17:34 davej Exp $
+ *  $Id: MSR-Athlon.c,v 1.5 2002/12/04 18:07:03 davej Exp $
  *  This file is part of x86info.
  *  (C) 2001 Dave Jones.
  *
@@ -20,11 +20,10 @@ void dump_athlon_MSR(struct cpudata *cpu)
 	if (!user_is_root)
 		return;
 
-	printf("\t\t\t\t31       23       15       7 \n");
 	dumpmsr(cpu->number, 0x2A, 32);
 	dumpmsr(cpu->number, 0xC0000080, 32);
 	dumpmsr(cpu->number, 0xC0010010, 32);
 	dumpmsr(cpu->number, 0xC0010015, 32);
-	dumpmsr(cpu->number, 0xC001001B, 32);
+	dumpmsr(cpu->number, MSR_CLKCTL, 32);
 	printf ("\n");
 }
