@@ -3,6 +3,9 @@
  *  This file is part of x86info.
  *  (C) 2001 Dave Jones.
  *
+ *  Copyright (C) 2007 Advanced Micro Devices, Inc.
+ *                     Andreas Herrmann <andreas.herrmann3@amd.com>
+ *
  *  Licensed under the terms of the GNU GPL License version 2.
  *
  *  AMD-specific information
@@ -12,37 +15,191 @@
 
 #include <stdio.h>
 #include <unistd.h>
+#include <string.h>
 #include <sys/types.h>
 #include "../x86info.h"
 #include "AMD.h"
+#include "revision.h"
 
 static char *amd_nameptr;
-#define add_to_cpuname(x)	amd_nameptr += snprintf(amd_nameptr, sizeof(x), "%s", x);
+#define add_to_cpuname(x) \
+do { \
+	amd_nameptr += snprintf(amd_nameptr, sizeof(x), "%s", x); \
+} while(0)
+
+static void set_k8_name(struct k8_rev *r, struct cpudata *c)
+{
+	int i, id, cont;
+	char s[CPU_NAME_LEN];
+	const char *p;
+
+	s[0] = 0;
+	cont = 0;
+	for (i=0; (r!=NULL) && (i<ARRAY_SIZE(k8_names)); i++) {
+		p = NULL;
+		id = 1<<i;
+		if (r->nameid & id)
+			p = get_k8_name(id);
+		if (p) {
+			if (cont)
+				strncat(s, "/", CPU_NAME_LEN-1);
+			else
+				cont = 1;
+			strncat(s, p, CPU_NAME_LEN-1);
+		}
+	}
+	if (r)
+		snprintf(c->name, CPU_NAME_LEN, "%s (%s)", s, r->rev);
+	else
+		snprintf(c->name, CPU_NAME_LEN, "Unknown CPU");
+}
+
+void set_k8_revinfo(int id, struct cpudata *c)
+{
+	int i;
+	struct k8_rev *r;
+
+	c->connector = 0;
+	r = NULL;
+	for (i=0; i<ARRAY_SIZE(k8_revisions); i++) {
+		if (k8_revisions[i].eax == id) {
+			r = &k8_revisions[i];
+			break;
+		}
+	}
+
+	set_k8_name(r, c);
+	c->connector = r ? r->socketid : 0;
+}
+
+void set_fam10h_revinfo(int id, struct cpudata *c)
+{
+	unsigned long eax, ebx, ecx, edx;
+	int pkg_id;
+	const char *p;
+	
+
+	p = get_fam10h_revision_name(id);
+	if (p)
+		snprintf(c->name, CPU_NAME_LEN,
+			 "Quad-Core/Dual-Core/Embedded Opteron (%s)", p);
+	else
+		snprintf(c->name, CPU_NAME_LEN, "Unknown CPU");
+
+	cpuid(c->number, 0x80000001, &eax, &ebx, &ecx, &edx);
+	pkg_id = (ebx >> 28) & 0xf;
+
+	switch (pkg_id) {
+	case 0:
+		c->connector = CONN_SOCKET_F_R2;
+		break;
+	case 1:
+		c->connector = CONN_SOCKET_AM2_R2;
+		break;
+	default:
+		c->connector = 0;
+	}
+}
 
 static void do_assoc(unsigned long assoc)
 {
-	if ((assoc & 0xff) == 255)
-		printf("Fully");
-	else
-		printf("%lu-way", assoc);
-	printf(" associative. ");
+	switch (assoc & 0xff) {
+	case 0x0:
+		break;
+	case 0x1:
+		printf("Direct mapped. ");
+		break;
+	case 0xff:
+		printf("Fully associative. ");
+		break;
+	default:
+		printf("%lu-way associative. ", assoc);
+	}
+}
+
+static void do_l2assoc(unsigned long assoc)
+{
+	unsigned long a;
+
+	a = 0;
+	switch (assoc) {
+	case 0x0:
+		printf("Disabled. ");
+		break;
+	case 0x1:
+		printf("Direct mapped. ");
+		break;
+	case 0xf:
+		printf("Fully associative. ");
+		break;
+	case 0x2:
+		a = 2;
+		break;
+	case 0x4:
+		a = 4;
+		break;
+	case 0x6:
+		a = 8;
+		break;
+	case 0x8:
+		a = 16;
+		break;
+	case 0xa:
+		a = 32;
+		break;
+	case 0xb:
+		a = 48;
+		break;
+	case 0xc:
+		a = 64;
+		break;
+	case 0xd:
+		a = 96;
+		break;
+	case 0xe:
+		a = 128;
+		break;
+	default:
+		break;
+	}
+
+	if (a)
+		printf("%lu-way associative. ", a);
 }
 
 static void decode_AMD_cacheinfo(struct cpudata *cpu)
 {
 	unsigned long eax, ebx, ecx, edx;
 
+	if ((cpu->eflags_edx & 1<<26) && cpu->maxei >= 0x80000019) {
+		/* 1GB page TLB info */
+		cpuid(cpu->number, 0x80000019, &eax, &ebx, &ecx, &edx);
+
+		printf("L1 Data TLB (1G):           ");
+		do_l2assoc(eax >> 28);
+		printf("%lu entries.\n", (eax >> 16) & 0xfff);
+		printf("L1 Instruction TLB (1G):    ");
+		do_l2assoc((eax >> 12) & 0xf);
+		printf("%lu entries.\n", eax & 0xfff);
+	}
+
 	if (cpu->maxei >= 0x80000005) {
 		/* TLB and cache info */
 		cpuid(cpu->number, 0x80000005, &eax, &ebx, &ecx, &edx);
 
-		printf("Instruction TLB: ");
-		do_assoc((ebx >> 8) & 0xff);
-		printf("%lu entries.\n", ebx & 0xff);
+		printf("L1 Data TLB (2M/4M):        ");
+		do_assoc(eax >> 24);
+		printf("%lu entries.\n", (eax >> 16) & 0xff);
+		printf("L1 Instruction TLB (2M/4M): ");
+		do_assoc((eax >> 8) & 0xff);
+		printf("%lu entries.\n", eax & 0xff);
 
-		printf("Data TLB: ");
+		printf("L1 Data TLB (4K):           ");
 		do_assoc(ebx >> 24);
 		printf("%lu entries.\n", (ebx >> 16) & 0xff);
+		printf("L1 Instruction TLB (4K):    ");
+		do_assoc((ebx >> 8) & 0xff);
+		printf("%lu entries.\n", ebx & 0xff);
 
 		printf("L1 Data cache:\n\t");
 		printf("Size: %luKb\t", ecx >> 24);
@@ -59,16 +216,54 @@ static void decode_AMD_cacheinfo(struct cpudata *cpu)
 		printf("line size=%lu bytes.\n", edx & 0xff);
 	}
 
-	/* check K6-III (and later) on-chip L2 cache size */
+	if ((cpu->eflags_edx & 1<<26) && cpu->maxei >= 0x80000019) {
+		/* 1GB page TLB info */
+		cpuid(cpu->number, 0x80000019, &eax, &ebx, &ecx, &edx);
+
+		printf("L2 Data TLB (1G):           ");
+		do_l2assoc(ebx >> 28);
+		printf("%lu entries.\n", (ebx >> 16) & 0xfff);
+		printf("L2 Instruction TLB (1G):    ");
+		do_l2assoc((ebx >> 12) & 0xf);
+		printf("%lu entries.\n", ebx & 0xfff);
+	}
+
 	if (cpu->maxei >= 0x80000006) {
+		/* K6-III (and later) on-chip L2 cache size */
 		cpuid(cpu->number, 0x80000006, &eax, &ebx, &ecx, &edx);
-		printf("L2 (on CPU) cache:\n\t");
+
+		printf("L2 Data TLB (2M/4M):        ");
+		do_l2assoc(eax >> 28);
+		printf("%lu entries.\n", (eax >> 16) & 0xfff);
+		printf("L2 Instruction TLB (2M/4M): ");
+		do_l2assoc((eax >> 12) & 0xf);
+		printf("%lu entries.\n", eax & 0xfff);
+
+		printf("L2 Data TLB (4K):           ");
+		do_l2assoc(ebx >> 28);
+		printf("%lu entries.\n", (ebx >> 16) & 0xfff);
+		printf("L2 Instruction TLB (4K):    ");
+		do_l2assoc((ebx >> 12) & 0xf);
+		printf("%lu entries.\n", ebx & 0xfff);
+
+		printf("L2 cache:\n\t");
 		printf("Size: %luKb\t", ecx >> 16);
-		do_assoc((ecx >> 12) & 0x0f);
+		do_l2assoc((ecx >> 12) & 0x0f);
 		printf("\n\t");
 		printf("lines per tag=%lu\t", (ecx >> 8) & 0x0f);
 		printf("line size=%lu bytes.\n", ecx & 0xff);
+		if (family(cpu) == 0x10) {
+			/* family 0x10 has shared L3  cache */
+			printf("L3 (shared) cache:\n\t");
+			printf("Size: %luKb\t",
+			       (edx >> 18) * 512);
+			do_l2assoc((edx >> 12) & 0x0f);
+			printf("\n\t");
+			printf("lines per tag=%lu\t", (edx >> 8) & 0x0f);
+			printf("line size=%lu bytes.\n", edx & 0xff);
+		}
 	}
+
 	printf("\n");
 }
 
@@ -133,15 +328,28 @@ void Identify_AMD(struct cpudata *cpu)
 	cpu->stepping = eax & 0xf;
 	cpu->model = (eax >> 4) & 0xf;
 	cpu->family = (eax >> 8) & 0xf;
-	cpu->emodel = (eax >> 16) & 0xff;
-	cpu->efamily= (eax >> 20) & 0xf;
+	if (cpu->family == 0xf) {
+		cpu->emodel = (eax >> 16) & 0xf;
+		cpu->efamily= (eax >> 20) & 0xff;
+	} else {
+		cpu->emodel = 0;
+		cpu->efamily = 0;
+	}
+
+	if (family(cpu) == 0xf) {
+		set_k8_revinfo(eax, cpu);
+		return;
+	} else if (family(cpu) == 0x10) {
+		set_fam10h_revinfo(eax, cpu);
+		return;
+	}
 
 	switch (cpu->family) {
 	case 4:
 		cpu->connector = CONN_SOCKET_3;
 		break;
 	}
-
+	
 	switch (tuple(cpu) & 0xff0) {
 	case 0x430:
 		add_to_cpuname("Am486DX2-WT");
@@ -182,10 +390,12 @@ void Identify_AMD(struct cpudata *cpu)
 		cpu->connector = CONN_SOCKET_5_7;
 		break;
 	case 0x560:
+		// 166,200,233 MHz
 		add_to_cpuname("K6 (0.30 um)");
 		cpu->connector = CONN_SOCKET_7;
 		break;
 	case 0x570:
+		// 200,233,266,300 MHz
 		add_to_cpuname("K6 (0.25 um)");
 		cpu->connector = CONN_SOCKET_7;
 		break;
@@ -451,161 +661,18 @@ out_660:
   * Registered DIMM not required
   * Athlon64 3x00+ (Q3 2004)
   */
-
-	case 0xF00:
-		cpu->connector = CONN_SOCKET_754;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 0:
-			add_to_cpuname("[SH7-A0]");
-			break;
-		case 1:
-			add_to_cpuname("[SH7-A2]");
-			break;
-		}
-		break;
-
-	case 0xF10:
-		add_to_cpuname("Opteron ES ");
-		cpu->connector = CONN_SOCKET_940;
-		switch (cpu->stepping) {
-		case 0:
-			add_to_cpuname("[SH7-A0]");
-			break;
-		case 1:
-			add_to_cpuname("[SH7-A2]");
-			break;
-		}
-		break;
-
-	case 0xF40:
-		cpu->connector = CONN_SOCKET_754;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 0:
-			if (cpu->emodel==0) {
-				add_to_cpuname("[SH7-B0]");
-			} else {
-				add_to_cpuname("[SH8-D0]");
-			}
-			break;
-		case 8:
-			// need to check for longmode bit. could be athlon xp 3000+.
-			//  (These are 32bit only amd64's)
-			//might be mobile
-			add_to_cpuname("[SH7-C0]");
-			break;
-		case 0xa:
-			//might be mobile
-			add_to_cpuname("[SH7-CG]");
-			break;
-		}
-		break;
-
-	// Gar, these could also be athlon 64fx
-	case 0xF50:
-		cpu->connector = CONN_SOCKET_940;
-		add_to_cpuname("Opteron");
-		switch (cpu->stepping) {
-		case 0:
-			if (cpu->emodel==0) {
-				add_to_cpuname("[SH7-B0]");
-			} else {
-				add_to_cpuname("[SH8-D0]");
-			}
-			break;
-		case 1:
-			add_to_cpuname("[SH7-B3]");
-			break;
-		case 8:
-			add_to_cpuname("[SH7-C0]");
-			break;
-		case 0xA:
-			add_to_cpuname("[SH7-CG]");
-			break;
-		default:
-			break;
-		}
-		break;
-
-	case 0xF70:
-		add_to_cpuname("Athlon 64 ");
-		cpu->connector = CONN_SOCKET_939;
-		switch (cpu->stepping) {
-		case 0x0:
-			add_to_cpuname("[SH8-D0]");
-			break;
-		case 0xa:
-			add_to_cpuname("[SH7-CG]");
-			break;
-		}
-		break;
-
-	case 0xF80:
-		cpu->connector = CONN_SOCKET_754;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 2:
-			//might be mobile
-			add_to_cpuname("CH7-CG");
-			break;
-		}
-		break;
-
-	case 0xFB0:
-		cpu->connector = CONN_SOCKET_939;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 2:
-			add_to_cpuname("CH7-CG");
-			break;
-		}
-		break;
-
-	case 0xFC0:
-		cpu->connector = CONN_SOCKET_754;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 0:
-			//might be mobile
-			add_to_cpuname("DH7-CG");
-			break;
-		}
-		break;
-
-	case 0xFE0:
-		//might be mobile
-		cpu->connector = CONN_SOCKET_754;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 0:
-			add_to_cpuname("DH7-CG");
-			break;
-		}
-		break;
-
-
-	case 0xFF0:
-		cpu->connector = CONN_SOCKET_939;
-		add_to_cpuname("Athlon 64 ");
-		switch (cpu->stepping) {
-		case 0:
-			add_to_cpuname("DH7-CG");
-			break;
-		}
-		break;
-
 	default:
 		add_to_cpuname("Unknown CPU");
 		break;
 	}
 }
 
-
 void display_AMD_info(struct cpudata *cpu)
 {
+	unsigned long eax, ebx, ecx, edx;
+
 	printf("Family: %u Model: %u Stepping: %u\n",
-	       cpu->family, cpu->model, cpu->stepping);
+	       family(cpu), model(cpu), cpu->stepping);
 	printf ("CPU Model : %s\n", cpu->name);
 	get_model_name(cpu);
 
@@ -630,16 +697,48 @@ void display_AMD_info(struct cpudata *cpu)
 	if (show_bugs)
 		show_amd_bugs(cpu);
 
+	if (cpu->maxi >= 0x05) {
+		cpuid (cpu->number, 0x05, &eax, &ebx, &ecx, NULL);
+		printf("Monitor/Mwait: min/max line size %d/%d%s%s\n",
+		       (int) (eax & 0xffff), (int) (ebx & 0xffff),
+		       (ecx & 0x2) ? ", ecx bit 0 support" : "",
+		       (ecx & 0x1) ? ", enumeration extension" : "");
+	}
+
+	if (cpu->maxei >= 0x8000000a) {
+		cpuid (cpu->number, 0x8000000a, &eax, &ebx, NULL, &edx);
+		printf("SVM: revision %d, %d ASIDs",
+		       (int) (eax & 0xff), (int) ebx);
+		if (edx & 1)
+			printf(", np");
+		if (edx & 2)
+			printf(", lbrVirt");
+		if (edx & 4)
+			printf(", SVMLock");
+		if (edx & 8)
+			printf(", NRIPSave");
+		printf("\n");
+	}
+
 	/* AMD Multicore characterisation */
 	if (cpu->maxei >= 0x80000008) {
-		int nr_cores;
-		unsigned long ecx;
+		int n, p;
+		cpuid (cpu->number, 0x80000008, &eax, NULL, &ecx, NULL);
+		printf("Address Size: %d bits virtual, %d bits physical\n",
+		       (int) (eax >> 8) & 0xff, (int) eax & 0xff);
 
-		cpuid (cpu->number, 0x80000008, NULL, NULL, &ecx, NULL);
-		nr_cores = 1 + (ecx & 0xff);
+		p = (ecx >> 12) & 0xf;
+		n = (ecx & 0xff) + 1;
+		if (p)
+			p = 1 << p;
+		else
+			p = n;
 
-		if (nr_cores > 1)
-			printf ("The physical package has %d cores\n", nr_cores);
+		if (p > 1)
+			printf("The physical package has %d of %d "
+			       "possible cores implemented.\n", n, p);
 	}
+
+
 }
 
