@@ -14,7 +14,7 @@
 static char *idt_nameptr;
 #define add_to_cpuname(x)   idt_nameptr += snprintf(idt_nameptr, sizeof(x), "%s", x);
 
-void Identify_IDT (struct cpudata *cpu)
+void Identify_IDT(struct cpudata *cpu)
 {
 	char *nameptr;
 	unsigned long eax, ebx, ecx, edx;
@@ -27,7 +27,7 @@ void Identify_IDT (struct cpudata *cpu)
 	if (cpu->maxi < 1)
 		return;
 
-	cpuid (cpu->number, 1, &eax, &ebx, &ecx, &edx);
+	cpuid(cpu->number, 1, &eax, &ebx, &ecx, &edx);
 	cpu->stepping = eax & 0xf;
 	cpu->model = (eax >> 4) & 0xf;
 	cpu->family = (eax >> 8) & 0xf;
@@ -121,11 +121,11 @@ void Identify_IDT (struct cpudata *cpu)
 		case 0x6A0:	switch (cpu->stepping) {
 					case 0:
 					case 8 ... 0xF:
-							add_to_cpuname("VIA C3 (Esther) [C7-M]");
-							break;
+						add_to_cpuname("VIA C3 (Esther) [C7-M]");
+						break;
 					case 1 ... 7:
-							add_to_cpuname("VIA C3 (Ruth) [C7-M]");
-							break;
+						add_to_cpuname("VIA C3 (Ruth) [C7-M]");
+						break;
 					}
 					break;
 
@@ -134,8 +134,9 @@ void Identify_IDT (struct cpudata *cpu)
 	// C5X
 	// CZA
 
-		default:	add_to_cpuname("Unknown VIA CPU");
-					break;
+		default:
+			add_to_cpuname("Unknown VIA CPU");
+			break;
 	}
 }
 
@@ -146,45 +147,47 @@ void decode_IDT_cacheinfo(struct cpudata *cpu)
 
 	if (cpu->maxei >= 0x80000005) {
 		/* TLB and cache info */
-		cpuid (cpu->number, 0x80000005, &eax, &ebx, &ecx, &edx);
-		printf ("Instruction TLB: %ld-way associative. %ld entries.\n", (ebx >> 8) & 0xff, ebx & 0xff);
-		printf ("Data TLB: %ld-way associative. %ld entries.\n", ebx >> 24, (ebx >> 16) & 0xff);
-		printf ("L1 Data cache:\n\tSize: %ldKb\t%ld-way associative.\n\tlines per tag=%ld\tline size=%ld bytes.\n",
-			ecx >> 24, (ecx >> 16) & 0xff, (ecx >> 8) & 0xff, ecx & 0xff);
-		printf ("L1 Instruction cache:\n\tSize: %ldKb\t%ld-way associative.\n\tlines per tag=%ld\tline size=%ld bytes.\n",
+		cpuid(cpu->number, 0x80000005, &eax, &ebx, &ecx, &edx);
+		printf("Cache info\n");
+		printf(" L1 Instruction cache: %ldKB, %ld-way associative, %ld lines per tag, line size=%ld bytes.\n",
 			edx >> 24, (edx >> 16) & 0xff, (edx >> 8) & 0xff, edx & 0xff);
+		printf(" L1 Data cache: %ldKB %ld-way associative, %ld lines per tag, line size=%ld bytes.\n",
+			ecx >> 24, (ecx >> 16) & 0xff, (ecx >> 8) & 0xff, ecx & 0xff);
+		if (cpu->maxei >= 0x80000006) {
+			cpuid (cpu->number, 0x80000006, &eax, &ebx, &ecx, &edx);
+			if ((cpu->family==6) && (cpu->model==7 || cpu->model==8))
+				/* Work around errata. */
+				printf (" L2 (on CPU) cache: %ldKB %ld-way associative, %ld lines per tag, line size=%ld bytes.\n",
+					ecx >> 24, (ecx >> 16) & 0x0f, (ecx >> 8) & 0x0f, ecx & 0xff);
+			else
+				printf (" L2 (on CPU) cache: %ldKB %ld-way associative, %ld lines per tag, line size=%ld bytes.\n",
+					ecx >> 16, (ecx >> 12) & 0x0f, (ecx >> 8) & 0x0f, ecx & 0xff);
+		}
+		printf("TLB info\n");
+		cpuid(cpu->number, 0x80000005, &eax, &ebx, &ecx, &edx);
+		printf(" Instruction TLB: %ld-way associative. %ld entries.\n", (ebx >> 8) & 0xff, ebx & 0xff);
+		printf(" Data TLB: %ld-way associative. %ld entries.\n", ebx >> 24, (ebx >> 16) & 0xff);
 	}
 
 	/* check on-chip L2 cache size */
-	if (cpu->maxei >= 0x80000006) {
-		cpuid (cpu->number, 0x80000006, &eax, &ebx, &ecx, &edx);
-		if ((cpu->family==6) && (cpu->model==7 || cpu->model==8)) {
-			printf ("L2 cache size errata detected. Using workaround\n");
-			printf ("L2 (on CPU) cache:\n\tSize: %ldKb\t%ld-way associative.\n\tlines per tag=%ld\tline size=%ld bytes.\n",
-				ecx >> 24, (ecx >> 16) & 0x0f, (ecx >> 8) & 0x0f, ecx & 0xff);
-		} else
-			printf ("L2 (on CPU) cache:\n\tSize: %ldKb\t%ld-way associative.\n\tlines per tag=%ld\tline size=%ld bytes.\n",
-				ecx >> 16, (ecx >> 12) & 0x0f, (ecx >> 8) & 0x0f, ecx & 0xff);
-	}
 }
 
 
 void display_IDT_info(struct cpudata *cpu)
 {
-	printf ("Family: %u Model: %u Stepping: %u\n",
+	printf("Family: %u Model: %u Stepping: %u\n",
 		cpu->family, cpu->model, cpu->stepping);
 	printf ("CPU Model : %s\n", cpu->name);
 	get_model_name (cpu);
 
 
 	/* Check for presence of extended info */
-	decode_feature_flags (cpu);
+	decode_feature_flags(cpu);
 
 	if (cpu->maxei == 0)
 		return;
 
-	if (show_cacheinfo)
-		decode_IDT_cacheinfo(cpu);
+	decode_IDT_cacheinfo(cpu);
 
 	if (cpu->family == 6 && show_registers)
 		dump_C3_MSR(cpu);
