@@ -370,7 +370,7 @@ void Identify_Intel (struct cpudata *cpu)
 	case 0x680:
 		switch (cpu->brand) {
 		case 2:
-			add_to_cpuname("Pentium III-M (Coppermine)");
+			add_to_cpuname("Pentium III/Pentium III-M (Coppermine)");
 			switch (cpu->stepping) {
 			case 1:
 				add_to_cpuname(" [cA2]");
@@ -662,7 +662,11 @@ void Identify_Intel (struct cpudata *cpu)
 		if(cpu->MHz/100 >= 29) {
 			add_to_cpuname("Extreme ");
 		} else {
-			add_to_cpuname("Duo ");
+			if (cpu->stepping == 7) {
+				add_to_cpuname("Quad (Kentsfield)");
+			} else {
+				add_to_cpuname("Duo ");
+			}
 		}
 		// Check for Thermal Monitor 2 feature bit, because only the
 		// non-mobile processors have it
@@ -720,13 +724,77 @@ void Identify_Intel (struct cpudata *cpu)
 		// The Sept 06 Core 2 Intel Errata documentation says there are
 		// at least B1 and B2 steppings.
 		switch(cpu->stepping) {
+		case 2:
+			// 2M L2
+			// SL9TB E4300 1.86GHz
+			// SLA3F E4400 2GHz
+			// SL9TA E6300 1.86GHz
+			// SL9T9 E6400 2.13GHz
+			add_to_cpuname(" [L2]");
+			break;
 		// TODO: B1 as stepping 5 is a 100% guess
 		case 5:
 			add_to_cpuname(" [B1]");
 			break;
 		case 6:
+			// 2M L2
+			// SL9SA E6300 1.86GHz
+			// SL9S9 E6400 2.13GHz
+			//
+			// 4M L2
+			// SLA4U E6320 1.86GHz
+			// SLA4T E6420 2.13GHz
+			// SL9S8 E6600 2.4GHz
+			// SL9ZL E6700 2.66GHz
+			// SL9ZF E6700 2.66GHz
+			// SL9S5 X6800 2.93GHz
 			add_to_cpuname(" [B2]");
 			break;
+		case 7:
+			// SL9UK [B3] 2.93GHz  QX6800
+			// SL9UL [B3] 2.66GHz  QX6700
+			// SL9UM [B3] 2.4GHz   Q6600
+			// SLACP [G0] 2.93GHz  QX6800
+			break;
+		case 0xa:
+			// sCode Procname (IDA/HFM)
+			// SLA43 T7700 (2.6GHz/2.4GHz)
+			// SLA3M T7700 (2.6GHz/2.4GHz)
+			// SLA44 T7500 (2.4GHz/2.2GHz)
+			// SLA3N T7500 (2.4GHz/2.2GHz)
+			// SLA45 T7300 (2.4GHz/2.0GHz)
+			// SLA3P T7300 (2.2GHz/2.0GHz)
+			// SLA3R L7500 (1.8GHz/1.6GHz)
+			// SLA3S L7300 (1.6GHz/1.4GHz)
+			// SLA33 X7900 (2.8GHz)
+			// SLA6Z X7800 (2.6GHz)
+			add_to_cpuname(" [E1]");
+			break;
+		case 0xb:
+			// SLAA5 2.33GHz E6540
+			// SLA9X 2.33GHz E6550
+			// SLA9V 2.66GHz E6750
+			// SLA9U 3GHz    E6850
+			// SLAFN 3GHz    QX6850
+			// SLACQ 2.66GHz Q6700
+			// SLACR 2.4GHz  Q6600
+			//
+			// Xeon 7200/7300 series
+			// SLA67 2.93GHz X7350 [8M L2]
+			// SLA68 2.4GHz  E7340 [8M L2]
+			// SLA77 2.4GHz  E7330 [8M L2]
+			// SLA69 2.13GHz E7320 [4M L2]
+			// SLA6A 1.6GHz  E7310 [4M L2]
+			// SLA6B 1.86GHz L7345 [8M L2]
+			// SLA6C 2.93GHz E7220 [8M L2]
+			// SLA6D 2.4GHz  E7210 [8M L2]
+			add_to_cpuname(" [G0]");
+			break;
+		case 0xd:
+			// SLA98 E4400 2GHz
+			// SLA95 E4500 2GHz
+			add_to_cpuname(" [M0]");
+			break;
 		}
 		break;
 
