@@ -5,8 +5,8 @@ void binary (unsigned int n, unsigned long value)
 	unsigned int i;
 
 	for(i=0; i<n; i++, value<<=1)
-		putchar( (1<<(n-1) & value) ? '1' : '0' );
-	putchar('\n');
+		(void)putchar( (1<<(n-1) & value) ? '1' : '0' );
+	(void)putchar('\n');
 }
 
 void binary32 (unsigned long value)
@@ -14,12 +14,12 @@ void binary32 (unsigned long value)
 	int i;
 
 	for(i=0;i<32;i++,value<<=1) {
-		putchar( (1<<31 & value) ? '1' : '0' );
+		(void)putchar( (1<<31 & value) ? '1' : '0' );
 
 		if(i==23 || i==15 || i==7)
-			putchar(' ');
+			(void)putchar(' ');
 	}
-	putchar('\n');
+	(void)putchar('\n');
 }
 
 void binary64(unsigned long long value)
