@@ -1,7 +1,6 @@
 /*
- *  $Id: features.c,v 1.26 2004/03/19 14:44:41 davej Exp $
  *  This file is part of x86info
- *  (C) 2001 Dave Jones.
+ *  (C) 2001-2006 Dave Jones.
  *
  *  Licensed under the terms of the GNU GPL License version 2.
  *
@@ -11,7 +10,22 @@
 #include <stdio.h>
 #include "x86info.h"
 
-void decode_feature_flags (struct cpudata *cpu)
+void flag_decode(unsigned long reg, const char *flags[])
+{
+	unsigned int i;
+
+	for (i=0; i<32; i++) {
+		if (reg & (1<<i)) {
+			if (flags[i])
+				printf(" %s", flags[i]);
+			else
+				printf(" [%u]", i);
+		}
+	}
+}
+
+
+void decode_feature_flags(struct cpudata *cpu)
 {
 	unsigned long eax, ebx, ecx, edx;
 	unsigned int i;
@@ -42,20 +56,20 @@ void decode_feature_flags (struct cpudata *cpu)
 		"CMOV instruction",
 		"Page Attribute Table",
 		"36-bit PSEs",
-		"Processor serial number",
+		"Processor serial number",	/* reserved on AMD */
 		"CLFLUSH instruction",
 		NULL,
-		"Debug Trace Store",
-		"ACPI via MSR",
+		"Debug Trace Store",		/* reserved on AMD */
+		"ACPI via MSR",			/* reserved on AMD */
 		"MMX support",
 		"FXSAVE and FXRESTORE instructions",
 		"SSE support",
 		"SSE2 support",
-		"CPU self snoop",
+		"CPU self snoop",		/* reserved on AMD */
 		"Hyper-Threading",
-		"Thermal Monitor",
+		"Thermal Monitor",		/* reserved on AMD */
 		NULL,
-		"Pending Break Enable"
+		"Pending Break Enable"		/* reserved on AMD */
 	};
 	/* CPUID 0x00000001 ECX flags */
 	const char *intel_cap_generic_ecx_flags[] = {
@@ -79,32 +93,41 @@ void decode_feature_flags (struct cpudata *cpu)
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 	};
 
-
 	const char *amd_cap_generic_ecx_flags[] = {
-		"sse3", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		"sse3", NULL, NULL, "mwait", NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, "cmpxchg16b", NULL, NULL,
-		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL, NULL, NULL, NULL, "popcnt",
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
 	};
 	const char *amd_cap_extended_edx_flags[] = {
-		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-		NULL, NULL, NULL, "syscall", NULL, NULL, NULL, NULL,
-		NULL, NULL, NULL, "mp/sempron", "nx", NULL, "mmxext", NULL,
-		NULL, "Fast fxsave/fxrstor", NULL, "rdtscp", NULL, "lm", "3dnowext", "3dnow"
-	};
+		"fpu", "vme", "de", "pse", "tsc", "msr", "pae", "mce",
+		"cx8", "apic", NULL, "sep", "mtrr", "pge", "mca", "cmov",
+		"pat", "pse36", NULL, "mp", "nx", NULL, "mmxext", "mmx",
+		"fxsr", "ffxsr", "page1gb", "rdtscp",
+		NULL, "lm", "3dnowext",	"3dnow"
+	}; /* "mp" defined for CPUs prior to AMD family 0xf */
+
 	const char *amd_cap_extended_ecx_flags[] = {
-		"lahf/sahf", "CMP legacy", "svm", "ExtApicSpace", "LockMovCr0", NULL, NULL, NULL,
-		"3DNowPrefetch", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		"lahf/sahf", "CmpLegacy", "svm", "ExtApicSpace",
+		"LockMovCr0", "abm", "sse4a", "misalignsse",
+		"3dnowPref", "osvw", "ibs", NULL, "skinit", "wdt", NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
 	};
 
-	const char *centaur_cap_flags[] = {
-		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+	const char *centaur_cap_extended_ecx_flags[] = {
+		"sse3", NULL, NULL, NULL, NULL, NULL, NULL, "EPS",
+		"tm2", NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, "mmxext", NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, "3dnowext", "3dnow"
 	};
+	const char *centaur_cap_extended_edx_flags[] = {
+		NULL, NULL, "RNGp", "RNGe", NULL, NULL, "ACEp", "ACEe",
+		"ACE2p", "ACE2e", "PHEp", "PHEe", "PMMp", "PMMe", NULL, NULL,
+		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
+	};
+
 	const char *transmeta_cap_flags[] = {
 		"recovery", "longrun", NULL, "lrti", NULL, NULL, NULL, NULL,
 		NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
@@ -125,53 +148,49 @@ void decode_feature_flags (struct cpudata *cpu)
 		return;
 
 	printf ("Feature flags:\n");
-	for (i = 0; i < 32; i++) {
+	for (i=0; i<32; i++) {
 		if (cpu->flags_edx & (1 << i)) {
-			if (verbose) {
+			if (!(generic_cap_flags_desc[i]))
+				printf(" [%u]", i);
+			if (verbose)
 				printf ("\t%s\n", generic_cap_flags_desc[i]);
-			} else {
+			else
 				printf (" %s", generic_cap_flags[i]);
-			}
 		}
 	}
 
 	/* Vendor specific extensions. */
 	switch (cpu->vendor) {
 		case VENDOR_AMD:
-			for (i = 0; i < 32; i++) {
+			for (i=0; i<32; i++) {
 				if (cpu->flags_ecx & (1 << i) && amd_cap_generic_ecx_flags[i])
 					printf (" %s", amd_cap_generic_ecx_flags[i]);
 			}
-			printf ("\n");
+			printf("\n");
 			if (cpu->maxei < 0x80000001)
 				break;
 			printf ("Extended feature flags:\n");
-			for (i = 0; i < 32; i++) {
-				if (cpu->eflags_edx & (1 << i) && amd_cap_extended_edx_flags[i])
-					printf (" %s", amd_cap_extended_edx_flags[i]);
-			}
-			for (i = 0; i < 32; i++) {
-				if (cpu->eflags_ecx & (1 << i) && amd_cap_extended_ecx_flags[i])
-					printf (" %s", amd_cap_extended_ecx_flags[i]);
-			}
+			flag_decode(cpu->eflags_edx, amd_cap_extended_edx_flags);
+			flag_decode(cpu->eflags_ecx, amd_cap_extended_ecx_flags);
+			printf("\n");
 			break;
 
 		case VENDOR_CENTAUR:
 			printf ("\n");
 			printf ("Extended feature flags:\n");
-			for (i = 0; i < 32; i++) {
-				if (cpu->flags_ecx & (1 << i) && centaur_cap_flags[i])
-					printf (" %s", centaur_cap_flags[i]);
+			flag_decode(cpu->flags_ecx, centaur_cap_extended_ecx_flags);
+			cpuid(cpu->number, 0xc0000000, &eax, &ebx, &ecx, &edx);
+			if (eax >=0xc0000001) {
+				cpuid(cpu->number, 0xc0000001, &eax, &ebx, &ecx, &edx);
+				cpu->flags_edx = edx;
+				flag_decode(cpu->flags_edx, centaur_cap_extended_edx_flags);
 			}
 			break;
 
 		case VENDOR_TRANSMETA:
 			printf ("\n");
 			printf ("Extended feature flags:\n");
-			for (i = 0; i < 32; i++) {
-				if (cpu->flags_ecx & (1 << i) && transmeta_cap_flags[i])
-					printf (" %s", transmeta_cap_flags[i]);
-			}
+			flag_decode(cpu->flags_ecx, transmeta_cap_flags);
 			break;
 
 		case VENDOR_CYRIX:
@@ -179,22 +198,14 @@ void decode_feature_flags (struct cpudata *cpu)
 			break;
 
 		case VENDOR_INTEL:
-			for (i = 0; i < 32; i++) {
-				if (cpu->flags_ecx & (1 << i) && intel_cap_generic_ecx_flags[i])
-					printf (" %s", intel_cap_generic_ecx_flags[i]);
-			}
 			printf ("\n");
+			printf ("Extended feature flags:\n");
+			flag_decode(cpu->flags_ecx, intel_cap_generic_ecx_flags);
 			if (cpu->maxei < 0x80000001)
 				break;
-			printf ("Extended feature flags:\n");
-			for (i = 0; i < 32; i++) {
-				if (cpu->eflags_edx & (1 << i) && intel_cap_extended_edx_flags[i])
-					printf (" %s", intel_cap_extended_edx_flags[i]);
-			}
-			for (i = 0; i < 32; i++) {
-				if (cpu->eflags_ecx & (1 << i) && intel_cap_extended_ecx_flags[i])
-					printf (" %s", intel_cap_extended_ecx_flags[i]);
-			}
+			printf ("\n");
+			flag_decode(cpu->eflags_edx, intel_cap_extended_edx_flags);
+			flag_decode(cpu->eflags_ecx, intel_cap_extended_ecx_flags);
 			break;
 
 		default:
