@@ -16,6 +16,56 @@
 #include <unistd.h>
 #include "x86info.h"
 
+#if defined(__FreeBSD__)
+# include <sys/ioctl.h>                                                         
+# include <cpu.h>
+#endif
+
+#if defined(__FreeBSD__)
+
+int read_msr(int cpu, unsigned int idx, unsigned long long *val)
+{
+	char cpuname[16];
+	unsigned char buffer[8];
+	unsigned long lo, hi;
+	int fh;
+	static int nodriver=0;
+	cpu_msr_args_t args;
+
+	if (nodriver==1)
+		return 0;
+
+	(void)snprintf(cpuname, sizeof(cpuname), "/dev/cpu%d", cpu);
+
+	fh = open(cpuname, O_RDONLY);
+	if (fh==-1) {
+		if (!silent)
+			perror(cpuname);
+		nodriver=1;
+		return 0;
+	}
+
+	args.msr = idx;                                                         
+	if (ioctl(fh, CPU_RDMSR, &args) != 0) {                                 
+		if (close(fh) == -1) {                                          
+			perror("close");                                        
+			exit(EXIT_FAILURE);
+		}
+
+		return 0;
+	}
+
+	*val = args.data; 
+
+	if (close(fh)==-1) {
+		perror("close");
+		exit(EXIT_FAILURE);
+	}
+	return 1;
+}
+
+#else /* !__FreeBSD__ */
+
 int read_msr(int cpu, unsigned int idx, unsigned long long *val)
 {
 	char cpuname[16];
@@ -63,6 +113,8 @@ int read_msr(int cpu, unsigned int idx, unsigned long long *val)
 	return 1;
 }
 
+#endif /* __FreeBSD__ */
+
 
 void dumpmsr (int cpu, unsigned int msr, int size)
 {
