@@ -1,6 +1,6 @@
 /*
  *  This file is part of x86info.
- *  (C) 2001-2006 Dave Jones.
+ *  (C) 2001-2007 Dave Jones.
  *
  *  Licensed under the terms of the GNU GPL License version 2.
  */
@@ -177,7 +177,7 @@ int main (int argc, char **argv)
 
 	parse_command_line(argc, argv);
 	if (!silent) {
-		printf ("x86info v1.20.  Dave Jones 2001-2006\n");
+		printf ("x86info v1.21.  Dave Jones 2001-2007\n");
 		printf ("Feedback to <davej@redhat.com>.\n\n");
 	}
 
@@ -193,7 +193,7 @@ int main (int argc, char **argv)
 	if (need_root && !user_is_root)
 		printf ("Need to be root to use specified options.\n");
 
-	nrCPUs = sysconf (_SC_NPROCESSORS_CONF);
+	nrCPUs = sysconf (_SC_NPROCESSORS_ONLN);
 
 	if (!silent) {
 		printf ("Found %u CPU", nrCPUs);
